// ppio.h

//;-------------------------------------------------------------------------
//; Copyright (c) 2002 Steven Freyder, David Helland, and Bruce D. Lightner.
//; Use subject to licensing restrictions described in "PICOWEB SERVER
//; SOFTWARE LICENSE AGREEMENT DATED JULY 14, 1999" (See file LICENSE.tt).
//;-------------------------------------------------------------------------

/******************************************************************************

  PicoWeb Parallel Port Programming Cable

  PC PP  Breadboard    PicoWeb
  DB-25  8-Pin Header  DB-25    Signal Name
  -----  ------------  -------  -----------
    2       JP1-1        20     RESET* (active low)
    3       JP1-3        18     SCK (PB7)
    4       JP1-7        14     MOSI (PB5)
   13       JP1-5        16     MISO (PB6)
   15       JP1-2         -     GND
   24       JP1-4         -     GND
   23       JP1-6         -     GND
   22       JP1-8         1     GND
   12         -           -     PaperOut      \ Loop-back (cable detect)
   14         -           -     AutoLineFeed* / Loop-back (cable detect)

******************************************************************************/

#ifndef __PPIO_INCLUDED__
#define __PPIO_INCLUDED__

#if !defined (LINUX)

#if defined(WIN32)
#include <stdio.h>

#include <conio.h>
//#include <dos.h>
#pragma intrinsic(_inp, _outp)

///#define DEBUG_LOW_LEVEL_IO
#ifndef DEBUG_LOW_LEVEL_IO
#define _pp_inp(x)    _inp((unsigned short)(x))
#define _pp_outp(x,y) _outp((unsigned short)(x),((int)(y)))
#else
static _inline int debug_pp_inp(int x)
{
    int y = _inp((unsigned short)x);
    printf("_pp_inp(0x%x) => 0x%02x\n", x, y & 0xff);
    fflush(stdout);
    return y;
}
static _inline void debug_pp_outp(int x, int y)
{
    printf("_pp_outp(0x%x,0x%x)\n", x, y & 0xff);
    fflush(stdout);
    _outp((unsigned short)x, (int)y);
}
#define _pp_inp(x) debug_pp_inp(x)
#define _pp_outp(x,y) debug_pp_outp(x,y)
#endif

#else /* !WIN32 */

#include <conio.h>
#include <dos.h>

#ifdef NOT_MSC_COMPAT
#define _pp_inp inportb
#define _pp_outp outportb
#else /* !NOT_MSC_COMPAT */
#define _pp_inp inp
#define _pp_outp outp
#endif /* !NOT_MSC_COMPAT */
#endif /* !WIN32 */

#else /* LINUX */

#include <unistd.h>

#ifndef NO_IO_INSTR
#include <asm/io.h>
#else /*!NO_IO_INSTR*/
void outb(int val,int port);
int inb(int port);
#endif /*!NO_IO_INSTR*/

#define _pp_inp(port) inb(port)
#define _pp_outp(port, value) outb(value, port)
#endif /* LINUX */

#define _PP_STATUS 1

#define _PP_RESETHI 0x1
#define _PP_SCKHI   0x2
#define _PP_MOSIHI  0x4
#define _PP_MISO    0x10

#define pp_read_miso() \
    ((!pp_invert_ios) ? \
        ((_pp_inp(_pp_io_base + _PP_STATUS) & _PP_MISO) ? 1 : 0) : \
        ((_pp_inp(_pp_io_base + _PP_STATUS) & _PP_MISO) ? 0 : 1) \
    )

#define pp_write(mosi, sck, reset) \
    { if (!pp_invert_ios) { \
        _pp_outp(_pp_io_base, ((mosi)?_PP_MOSIHI:0)|((sck)?_PP_SCKHI:0)|((reset)?_PP_RESETHI:0)); \
    } else { \
        int val = ((mosi)?_PP_MOSIHI:0)|((sck)?_PP_SCKHI:0)|((reset)?_PP_RESETHI:0); \
        _pp_outp(_pp_io_base, ~val); \
    } }

extern unsigned int _pp_io_base;
extern int directIO;
extern int pp_invert_ios;

void pp_set_hits_per_msec(long hits_per_msec);
long pp_get_hits_per_msec(void);

unsigned int pp_get_port(void);  // return LPT I/O port (0 = none)

int pp_get_lpt(void);  // return LPT number (-1=none, n=LPTn)

void pp_set_invert_ios(void);

int pp_delay_ms(int msecs);

int pp_set_lpt(      // returned LPT number (-1=not found; n=LPTn)
    FILE *fd,        // log errors and messages here
    int verbose,     // 0=only log errors; 1=log progress reports too, 2=debug
    int ecp_mode,    // PC parallel port type: 0:legacy, 1:ECP
    int port         // <0: search for cable; >0: check LPTn for cable
);

void pp_cleanup(
    int reset_active,  // make reset active
    int bus_active     // leave parallel port output drivers active
);

void pp_reset(void);

#endif /*__PPIO_INCLUDED__*/
